from astropy.units import (
    Quantity,
    W,  # pyright: ignore[reportAttributeAccessIssue]
    mW,  # pyright: ignore[reportAttributeAccessIssue]
    s,  # pyright: ignore[reportAttributeAccessIssue]
    Unit,
)
from astropy.units.physical import time
from gwpy.time import Time, from_gps
from numpy import array, where, diff

from ..component import Component
from ..lockingstate import LockingState
from ..measurement import Measurement
from ..scatterer import Scatterer
from ..utils import download_dicttimeseries, download_timeseries

from ..metadata.measurement import Measurement as MetadataMeasurement
from ..metadata.component import Component as MetadataComponent
from ..metadata.scatterer import Scatterer as MetadataScatterer

from ..factory.component import ComponentFactory
from ..factory.scatterer import ScattererFactory
from ..factory.initializer import (
    InitializerScatterer,
)


class MeasurementFactory:
    """
    Factory class to create Measurement
    """

    @staticmethod
    def fetch(
        name: str,
        description: str,
        start_time: Time,
        duration: Quantity[time],
        initializer_metadata: list[InitializerScatterer],
        force_virgotools: bool = False,
    ) -> Measurement:
        """
        Fetch a measurement from online data

        Construct a Measurement object, containing all the data of this
        measurement
        Parameters
        ----------
        name : str
            Measurement name
        description : str
            Detailed description of the created measurement
        start_time : gwpy.Time
            Starting time of the created measurement
        duration : astropy.units.Quantity[time]
            Duration of the created measurement
        initializer_metadata : list[InitializerScatterer]
            List of component's and scatterer's pre-initialized
            metadata that will be used in the created measurement
        force_virgotools : bool
            Force the use of the virgotools library in place of gwpy
            first to be faster. Default to False
        Examples
        --------
        >>> data = MeasurementFactory.fetch(
        ...     name="a name",
        ...     description="a detailed description",
        ...     start_time=Time(
        ...         "2025-01-01T00:00:00.000000000",
        ...         format="isot",
        ...         scale="utc",
        ...     ),
        ...     duration=10 * s,
        ...     initializer_metadata=[
        ...         InitializerScatterer(
        ...             name="SDB1",
        ...             calibration_factor=1.0 * m / V,
        ...             associated_component=InitializerComponent(
        ...                 name="SR",
        ...                 calibration_factor=1.0 * m / V,
        ...             ),
        ...             injection=None,
        ...         ),
        ...     ],
        ... )
        Returns
        -------
        Measurement : constructed Measurement from these metadata
        """
        channels_map: dict[str, Unit | None] = {
            "V1:META_ITF_LOCK_index": None,
            "V1:INJ_IMC_TRA_DC_mean": W,
            "V1:DET_B1p_DC_mean": mW,
        }
        data = download_dicttimeseries(
            channels=list(channels_map.keys()),
            start_time=start_time,
            duration=duration,
            units=list(channels_map.values()),
            frame="trend",
            force_virgotools=force_virgotools,
        )
        state_data = array(data["V1:META_ITF_LOCK_index"].data).astype(
            int
        )
        if sum(abs(diff(state_data))) != 0:
            indexes = where(diff(state_data) != 0)[0]
            times_change = (
                data["V1:META_ITF_LOCK_index"].t0
                + indexes * data["V1:META_ITF_LOCK_index"].dt
            )
            message = (
                "state of the interferometer changed in '{}'".format(
                    name
                )
            )
            for i in range(len(indexes)):
                time_change = times_change[i]
                index = int(indexes[i])
                message += "\nat {}, from {} to {}".format(
                    from_gps(time_change.to(s).value),
                    LockingState(state_data[index]).name,
                    LockingState(state_data[index + 1]).name,
                )
            raise Exception(message)
        state = LockingState(state_data[0])
        injection_power = data["V1:INJ_IMC_TRA_DC_mean"].mean()
        dark_fringe_power = data["V1:DET_B1p_DC_mean"].mean()

        metadata = MetadataMeasurement(
            name=name,
            description=description,
            state=state,
            dark_fringe_power=dark_fringe_power,
            injection_power=injection_power,
            start_time=start_time,
            duration=duration,
        )

        components: list[Component] = []
        scatterers: list[Scatterer] = []
        for initializer_scatterer in initializer_metadata:
            component: Component | None = None
            if initializer_scatterer.associated_component is not None:
                component = ComponentFactory.fetch(
                    start_time=start_time,
                    duration=duration,
                    metadata=MetadataComponent(
                        name=initializer_scatterer.associated_component.name,
                        calibration_factor=initializer_scatterer.associated_component.calibration_factor,
                    ),
                    force_virgotools=force_virgotools,
                )
                components.append(component)
            scatterers.append(
                ScattererFactory.fetch(
                    start_time=start_time,
                    duration=duration,
                    metadata=MetadataScatterer(
                        name=initializer_scatterer.name,
                        calibration_factor=initializer_scatterer.calibration_factor,
                        associated_component=component,
                        injection=initializer_scatterer.injection,
                    ),
                )
            )
        sensitivity = download_timeseries(
            channel="V1:Hrec_hoft_20000Hz",
            start_time=start_time,
            duration=duration,
            unit=None,
            force_virgotools=force_virgotools,
        )
        return Measurement(
            metadata=metadata,
            scatterers=scatterers,
            components=components,
            sensitivity=sensitivity,
        )
