from astropy.units import Quantity
from astropy.units.physical import length, electrical_potential
from h5py import File, Group

from .metadata import Metadata


class Component(Metadata):
    """
    Metadata associated to a vda.package.Component
    """

    def __init__(
        self,
        name: str,
        calibration_factor: Quantity[length / electrical_potential],
    ):
        """
        Parameters
        ----------
        name : str
            Unique name of the component
        calibration_factor : astropy.units.Quantity[ length / electrical potential ]
            Calibration factor to correct movement data. Movement
            sensor give movement in Volt, and have to be corrected by a
            given factor, so that `real movement = calibration_factor *
            sensor data`
        Warnings
        --------
        It's an internal class, of whose instances are not produced via
        a direct call, but by using helper functions
        See Also
        --------
        bed.Component : Component class containing this metadata
        Examples
        --------
        >>> component_metadata = bed.MetadataComponent(
        ...     "BS",
        ...     1.0 * u.m / u.V,
        ... )
        """
        self.name: str = name
        self.calibration_factor: Quantity[
            length / electrical_potential
        ] = calibration_factor

    @staticmethod
    def from_hdf5(store: File | Group) -> "Component":
        for attribute in ["name", "calibration factor"]:
            if attribute not in store.attrs.keys():
                raise ValueError(
                    "no {} for an interferometer's component".format(
                        attribute
                    )
                )
        name = store.attrs["name"]
        calibration_factor = store.attrs["calibration factor"]

        error = (
            "interferometer's component {} should be a string, not {}"
        )
        if not isinstance(name, str):
            raise ValueError(error.format("name", type(name)))
        if not isinstance(calibration_factor, str):
            raise ValueError(
                error.format(
                    "calibration factor", type(calibration_factor)
                )
            )

        calibration_factor = Quantity(calibration_factor)

        return Component(
            name=name,
            calibration_factor=calibration_factor,
        )

    def to_hdf5(self, store: File | Group) -> Group | File:
        store.attrs["name"] = self.name
        store.attrs["calibration factor"] = (
            self.calibration_factor.to_string()
        )
        return store
