# pyright: reportExplicitAny=false
from typing import final, Any
from astropy.units import Quantity
from astropy.units.physical import frequency, dimensionless
from h5py import File, Group

from .metadata import Metadata


class Injection(Metadata):
    """
    Metadata associated to an artificial injection on a component
    position.
    Warnings
    --------
    This class is never instantiated as is
    See Also
    --------
    bed.metadata.scatterer : scatterer's metadata, containing injection
    """

    @staticmethod
    def from_hdf5(store: File | Group) -> "Injection":
        """
        Warnings
        --------
        We assume that every attribute of an injection is a Quantity
        Returns
        -------
        It return a subclass of Injection, corresponding to the type
        specified in the file
        """
        if "type" not in store.attrs.keys():
            raise ValueError("injection type must be defined")
        type_ = store.attrs.keys()
        if not isinstance(type_, str):
            raise ValueError(
                "injection type must be a string, not {}".format(
                    type(type_)
                )
            )
        attributes: list[
            Quantity
        ] = []  # we force all injection attribute to be Quantity represented as a string
        for name, value in store.attrs.items():
            if name == "type":  # we already counted it
                continue
            if isinstance(value, str):
                attributes.append(Quantity(value))
        return InjectionFactory.make(type_, *attributes)


@final
class SinInjection(Injection):
    """
    Sine injection
    """

    def __init__(
        self,
        amplitude: Quantity[dimensionless],
        frequency: Quantity[frequency],
    ):
        """
        Parameters
        ----------
        amplitude : Quantity[dimensionless]
            Amplitude of the sine.
        frequency : Quantity[frequency]
            Frequency of the sine
        Note
        ----
            injected line = amplitude * sin( 2 * pi * frequency )
        """
        self.amplitude = amplitude
        self.frequency = frequency

    def to_hdf5(self, store: File | Group) -> Group | File:
        store.attrs["type"] = "sin"
        if type(self.amplitude) is Quantity:
            store.attrs["amplitude"] = self.amplitude.to_string()
        elif type(self.amplitude) is float:  # allowed (dimensionless)
            store.attrs["amplitude"] = self.amplitude
        else:
            raise ValueError(
                "amplitude must be a dimensionless quantity, not {}".format(
                    type(self.amplitude)
                )
            )
        store.attrs["frequency"] = self.frequency.to_string()
        return store


@final
class InjectionFactory:
    @staticmethod
    def make(type_: str, *args: Any, **kwargs: Any):
        """
        Return the "good" injection based on the first parameter
        Parameters
        ----------
        type_ : str
            Type of the injection
        *args : Any
            Parameter for the injection's constructor
        **kwargs : Any
            Parameter for the injection's constructor
        Returns
        -------
        bed.Injection : a child of the Injection class
        """
        if type_ == "sin":
            return SinInjection(*args, **kwargs)
        raise ValueError(
            "unknown injection type {type}".format(type=type_)
        )
