from typing import final
from h5py import File, Group
from astropy.units import Quantity
from astropy.units.physical import power, time
from gwpy.time import Time, to_gps

from ..lockingstate import LockingState
from .metadata import Metadata


@final
class Measurement(Metadata):
    """
    Metadata associated to a vda.package.Measurement
    """

    def __init__(
        self,
        name: str,
        description: str,
        state: LockingState,
        dark_fringe_power: Quantity[power],
        injection_power: Quantity[power],
        start_time: Time,
        duration: Quantity[time],
    ):
        """
        Parameters
        ----------
        name : str
            Unique name of the measurement
        description : str
            Detailed description of the measurement
        state : vda.LockingState
            Lock index when this measurement was done
        dark_fringe_power : Quantity[power]
            Power on the dark fringe (dark fringe offset)
        injection_power : Quantity[power]
            Power injected in the interferometer
        start_time : gwpy.time.LIGOTimeGPS
            Start GPS time
        duration : Quantity[time]
            Duration of the measurement
        Warnings
        --------
        It's an internal class, of whose instances are not produced via
        a direct call, but by using helper functions
        See Also
        --------
        bed.Measurement : Component class containing this metadata
        Examples
        --------
        >>> measurement_metadata = vda.package.metadata.Measurement(
        ...     "testing SDB1 with SR aligned",
        ...     "Check how fsc evolve if SR is aligned, which is the case in LOW_NOISE_2",
        ...     8 * u.mW,
        ...     17.9 * u.W,
        ...     gwpy.time.to_gps("17-06-2025 18:22:50"),
        ...     4 * u.min,
        ... )
        """
        self.name = name
        self.description = description
        self.state = state
        self.dark_fringe_power = dark_fringe_power
        self.injection_power = injection_power
        self.start_time = start_time
        self.duration = duration

    @staticmethod
    def from_hdf5(store: File | Group) -> "Measurement":
        for attribute in [
            "name",
            "description",
            "state",
            "dark fringe power",
            "injection power",
            "start time",
            "duration",
        ]:
            if attribute not in store.attrs.keys():
                raise ValueError(
                    "no {} for a measurement".format(attribute)
                )

        name = store.attrs["name"]
        description = store.attrs["description"]
        state = store.attrs["state"]
        dark_fringe_power = store.attrs["dark fringe power"]
        injection_power = store.attrs["injection power"]
        start_time = store.attrs["start time"]
        duration = store.attrs["duration"]

        error = "measurement {} should be a {}, not {}"
        if not isinstance(name, str):
            raise ValueError(error.format("name", "string", type(name)))
        if not isinstance(description, str):
            raise ValueError(
                error.format("description", "string", type(description))
            )
        if not isinstance(state, str):
            raise ValueError(
                error.format("state", "string", type(state))
            )
        if not isinstance(dark_fringe_power, str):
            raise ValueError(
                error.format(
                    "dark fringe power",
                    "str",
                    type(dark_fringe_power),
                )
            )
        if not isinstance(injection_power, str):
            raise ValueError(
                error.format(
                    "injection power",
                    "str",
                    type(injection_power),
                )
            )
        if not isinstance(start_time, int):
            raise ValueError(
                error.format("start time", "int", type(start_time))
            )
        if not isinstance(duration, str):
            raise ValueError(
                error.format("duration", "float64", type(duration))
            )

        try:
            state = LockingState[state]
        except KeyError:
            raise ValueError(
                "locking state {} is not valid".format(state)
            )
        dark_fringe_power = Quantity(dark_fringe_power)
        injection_power = Quantity(injection_power)
        start_time = Time(start_time, format="gps")
        duration = Quantity(duration)

        return Measurement(
            name=name,
            description=description,
            state=state,
            dark_fringe_power=dark_fringe_power,
            injection_power=injection_power,
            start_time=start_time,
            duration=duration,
        )

    def to_hdf5(self, store: File | Group) -> Group | File:
        store.attrs["name"] = self.name
        store.attrs["description"] = self.description
        store.attrs["state"] = self.state.name
        store.attrs["dark fringe power"] = (
            self.dark_fringe_power.to_string()
        )
        store.attrs["injection power"] = (
            self.injection_power.to_string()
        )
        store.attrs["start time"] = float(to_gps(self.start_time))
        store.attrs["duration"] = self.duration.to_string()
        return store
