from typing import final
from h5py import File, Group
from .metadata import Metadata


@final
class Root(Metadata):
    """
    Metadata associated to a bed.Root
    """

    def __init__(self, name: str, description: str):
        """
        Parameters
        ----------
        name : str
            Unique name of the package
        description : str
            Detailed description of the package: why reuniting these
            measurements into this particular group ?
        Warnings
        --------
        It's an internal class, of whose instances are not produced via
        a direct call, but by using helper functions
        See Also
        --------
        bed.Root : Root class containing this metadata
        Examples
        --------
        >>> package_metadata = bed.MetadataRoot(
        ...     "june measurements 2025",
        ...     "All the measurement done during the shift in the evening of 17/07/2025",
        ... )
        """
        self.name = name
        self.description = description

    @staticmethod
    def from_hdf5(store: File | Group) -> "Root":
        if "name" not in store.attrs.keys():
            raise ValueError("no name for the experimental data")
        if "description" not in store.attrs.keys():
            raise ValueError("no description for the experimental data")

        name = store.attrs["name"]
        description = store.attrs["description"]
        if not isinstance(name, str):
            raise ValueError(
                "experimental data name should be a string, not {}".format(
                    type(name)
                )
            )
        if not isinstance(description, str):
            raise ValueError(
                "experimental data description should be a string, not {}".format(
                    type(description)
                )
            )
        return Root(
            name=name,
            description=description,
        )

    def to_hdf5(self, store: File | Group) -> Group | File:
        store.attrs["name"] = self.name
        store.attrs["description"] = self.description
        return store
