from typing import final

from astropy.units import Quantity
from astropy.units.physical import length, electrical_potential
from h5py import File, Group

from .injection import Injection
from ..component import Component
from .component import Component as MetadataComponent


@final
class Scatterer(MetadataComponent):
    """
    Metadata associated to a vda.package.Scatterer
    """

    def __init__(
        self,
        name: str,
        calibration_factor: Quantity[length / electrical_potential],
        associated_component: Component | None = None,
        injection: Injection | None = None,
    ):
        """
        Parameters
        ----------
        name : str
            Unique name of the scatterer
        calibration_factor : astropy.units.Quantity[ length / electrical potential ]
            Calibration factor to correct movement data. Movement
            sensor give movement in Volt, and have to be corrected by a
            given factor, so that `real movement = calib_factor *
            sensor data`
        associated_component : vda.package.Component
            An interferometer's component that is close enough to the
            scatterer that the background noise in the movement sensor
            if well correlated. Optional, but strongly encouraged as the
            movement will be contamined by noise. Default to None.
        injection : vda.package.metadata.Injection
            A metadata containing information about an artificial
            injection on this component. None if there was no injection
            on this bench. Optional, default to None.
        Warnings
        --------
        It's an internal class, of whose instances are not produced via
        a direct call, but by using helper functions
        See Also
        --------
        bed.Scatterer : Component class containing this metadata
        Examples
        --------
        >>> scatterer_metadata = bed.MetadataScatterer(
        ...     "SDB1",
        ...     1.0 * u.m / u.V,
        ...     bed.Component(
        ...         bed.MetadataComponent(
        ...             "BS",
        ...             1.0 * u.m / u.V,
        ...         ),
        ...         data,
        ...     ),
        ...     bed.SinInjection(
        ...         100 * u.dimensionless_unscaled,
        ...         1 * u.Hz,
        ...     ),
        ... )
        """
        super().__init__(
            name=name, calibration_factor=calibration_factor
        )
        self.associated_component = associated_component
        self.injection = injection

    @staticmethod
    def from_hdf5(
        store: File | Group, components: list[Component] | None = None
    ) -> "Scatterer":
        injection_group = store.get("injection")
        if not isinstance(injection_group, Group):
            injection = None
        else:
            injection = Injection.from_hdf5(injection_group)
        for attribute in [
            "name",
            "calibration factor",
        ]:
            if attribute not in store.attrs.keys():
                raise ValueError(
                    "no {} for an interferometer's scatterer".format(
                        attribute
                    )
                )
        name = store.attrs["name"]
        calibration_factor = store.attrs["calibration factor"]

        associated_component_name = None
        if "associated component" in store.attrs.keys():
            associated_component_name = store.attrs[
                "associated component"
            ]

        error = "interferometer's scatterer {} should be string, not {}"
        if not isinstance(name, str):
            raise ValueError(error.format("name", type(name)))
        if not isinstance(calibration_factor, str):
            raise ValueError(
                error.format(
                    "calibration factor", type(calibration_factor)
                )
            )
        if not isinstance(associated_component_name, str):
            raise ValueError(
                error.format(
                    "associated component",
                    type(associated_component_name),
                )
            )

        calibration_factor = Quantity(calibration_factor)
        associated_component = None
        if components is not None:
            for component in components:
                if component.metadata.name == associated_component_name:
                    associated_component = component

        return Scatterer(
            name=name,
            calibration_factor=calibration_factor,
            associated_component=associated_component,
            injection=injection,
        )

    def to_hdf5(self, store: File | Group) -> Group | File:
        store.attrs["name"] = self.name
        store.attrs["calibration factor"] = (
            self.calibration_factor.to_string()
        )
        if self.injection is not None:
            _ = self.injection.to_hdf5(store.create_group("injection"))
        if self.associated_component is not None:
            store.attrs["associated component"] = (
                self.associated_component.metadata.name,
            )
        return store
