from typing import final
from gwpy.timeseries import TimeSeries
from h5py import Dataset, File, Group

from .section import Section
from .metadata.scatterer import Scatterer as Metadata
from .component import Component
from .utils import dataset_to_timeseries, timeseries_to_dataset


@final
class Scatterer(Section):
    """
    A Scatterer is an interferometer's component that is considered as a
    source of backscattering.
    Attributes
    ----------
    movement : gwpy.timeseries.TimeSeries[length]
        Corrected movement of the scatterer
    See Also
    --------
    vda.package.Component : An interferometer's component
    """

    def __init__(self, metadata: Metadata, sensor: TimeSeries):
        """
        Parameters
        ----------
        metadata : bed.MetadataScatterer
            Metadata associated to this data
        sensor : gwpy.timeseries.TimeSeries[Volt]
            Sensor data of the component (in Volt, not in meter !)
        Warnings
        --------
        It's an internal class, of whose instances are not produced via
        a direct call, but by using helper functions
        See Also
        --------
        bed.Scatterer : Associated metadata type
        """
        self.metadata = metadata
        self.sensor = sensor
        self.movement = self.sensor * self.metadata.calibration_factor

    @staticmethod
    def from_hdf5(
        store: File | Group, components: list[Component] | None = None
    ) -> "Scatterer":
        sensor_dataset = store.get("sensor")
        if not isinstance(sensor_dataset, Dataset):
            raise ValueError(
                "sensor should exists, and not be {}".format(
                    type(sensor_dataset)
                )
            )
        sensor = dataset_to_timeseries(sensor_dataset)
        return Scatterer(
            Metadata.from_hdf5(store, components), sensor=sensor
        )

    def to_hdf5(self, store: File | Group) -> Group | File:
        _ = self.metadata.to_hdf5(store)
        _ = timeseries_to_dataset(store, "sensor", self.sensor)
        return store
