from typing import Literal
from astropy.units import Quantity, Unit, s, Hz  # pyright: ignore[reportAttributeAccessIssue]
from astropy.units.physical import time
from h5py import Dataset, File, Group
from requests import HTTPError

from gwpy.time import Time, to_gps
from gwpy.timeseries import TimeSeries, TimeSeriesDict

try:
    from virgotools.frame_lib import FrameFile, FrVect  # pyright: ignore[reportMissingImports]

    virgotools = True
except ImportError:
    virgotools = False
    pass


def dataset_to_timeseries(dataset: Dataset) -> TimeSeries:
    """
    Convert a h5df dataset to a TimeSeries keeping the units right
    Parameters
    ----------
    dataset : Dataset
        Dataset to convert
    Returns
    -------
    TimeSeries : TimeSeries with a good unit
    """
    unit = None
    if "units" in dataset.attrs.keys():
        units = dataset.attrs["units"]
        if isinstance(units, str):
            unit = Unit(units)
        else:
            raise ValueError(
                "cannot convert {} to a unit".format(units)
            )
    if "t0" not in dataset.attrs.keys():
        raise ValueError("no starting time defined for this timeseries")
    if "dt" not in dataset.attrs.keys():
        raise ValueError(
            "no time sample separation defined for this timeseries"
        )
    t0 = (dataset.attrs["t0"],)
    dt = dataset.attrs["dt"]
    if not isinstance(t0, str):
        raise ValueError(
            "t0 must be stored as string, not {}".format(type(t0))
        )
    if not isinstance(dt, str):
        raise ValueError(
            "dt must be stored as string, not {}".format(type(dt))
        )
    t0 = Quantity(t0)
    dt = Quantity(dt)
    return TimeSeries(
        data=dataset[1],
        unit=unit,
        t0=t0,
        dt=dt,
    )


def timeseries_to_dataset(
    store: File | Group, name: str, timeseries: TimeSeries
) -> Dataset:
    """
    Convert a gwpy TimeSeries to a hdf5 dataset keeping the units right
    Parameters
    ----------
    store : File | Group
        hdf5 element which will store the Dataset
    name : str
        Name of the dataset in the store
    timeseries : TimeSeries
        Data to store in the Dataset
    Returns
    -------
    Dataset : Created dataset
    """
    dataset = store.create_dataset(name, data=timeseries.data)
    if timeseries.unit is not None:
        dataset.attrs["units"] = timeseries.unit.to_string()
    if not isinstance(timeseries.t0, Quantity):
        raise ValueError(
            "t0 must be a quantity, not {}".format(type(timeseries.t0))
        )
    if not isinstance(timeseries.dt, Quantity):
        raise ValueError(
            "t0 must be a quantity, not {}".format(type(timeseries.dt))
        )
    dataset.attrs["t0"] = timeseries.t0.to_string()
    dataset.attrs["dt"] = timeseries.dt.to_string()
    return dataset


def download_timeseries(
    channel: str,
    start_time: Time,
    duration: Quantity[time],
    unit: Unit | None = None,
    frame: Literal["raw"] | Literal["trend"] = "raw",
    force_virgotools: bool = False,
) -> TimeSeries:
    """
    Download a TimeSeries from an online channel
    Parameters
    ----------
    channel : str
        Channel name
    start_time : Time
        Starting time
    duration : Quantity[time]
        Duration
    unit : Unit | None
        Unit of the TimeSeries (default ot None)
    frame : raw|trend
        Which frame to take the data from. Default to raw
    force_virgotools : bool
        Force the use of the virgotools library in place of gwpy first
        to be faster. Default to False
    Notes
    -----
    It first try to use gwpy, and fallback to virgotools
    """
    if not force_virgotools:
        try:
            return TimeSeries.get(
                channel=channel,
                start=to_gps(start_time),
                end=to_gps(start_time + duration),
            )
        except HTTPError:
            pass
    if virgotools:
        vect: FrVect | None = None  # pyright: ignore[reportPossiblyUnboundVariable]
        with FrameFile(frame) as ffl:  # pyright: ignore[reportPossiblyUnboundVariable]
            vect = ffl.getChannel(  # pyright: ignore[reportPossiblyUnboundVariable]
                channel,
                to_gps(start_time),
                duration.to(s).value,
            )
        if vect is None:  # pyright: ignore[reportPossiblyUnboundVariable]
            raise RuntimeError(
                "cannot get data vector from online data"
            )
        return TimeSeries(
            data=vect.data,  # pyright: ignore[reportPossiblyUnboundVariable]
            unit=unit,
            t0=start_time,
            sample_rate=vect.fsample * Hz,  # pyright: ignore[reportPossiblyUnboundVariable]
        )
    else:
        raise RuntimeError(
            "virgotools not available in this environment, cannot download online Virgo data"
        )


def download_dicttimeseries(
    channels: list[str],
    start_time: Time,
    duration: Quantity[time],
    units: list[Unit | None] | None = None,
    frame: Literal["raw"] | Literal["trend"] = "raw",
    force_virgotools: bool = False,
) -> TimeSeriesDict:
    """
    Download multiple TimeSeries at the same time to be more efficient
    Parameters
    ----------
    channels : list[str]
        List of channel name
    start_time : Time
        Starting time
    duration : Quantity[time]
        Duration
    units : list[Unit | None] | None
        Units of the different channels. If None, no unit for all
        channels. Default to None
    frame : raw|trend
        Which frame to take the data from. Default to raw
    force_virgotools : bool
        Force the use of the virgotools library in place of gwpy first
        to be faster. Default to False
    Notes
    -----
    It first try to use gwpy, and fallback to virgotools
    """
    if not force_virgotools:
        try:
            return TimeSeriesDict.get(
                channels=channels,
                start=to_gps(start_time),
                end=to_gps(start_time + duration),
            )
        except HTTPError:
            pass
    if virgotools:
        vects: list[FrVect] = []  # pyright: ignore[reportPossiblyUnboundVariable]
        with FrameFile(frame) as ffl:  # pyright: ignore[reportPossiblyUnboundVariable]
            for channel in channels:
                vects.append(
                    ffl.getChannel(
                        channel,
                        to_gps(start_time),
                        duration.to(s).value,
                    )
                )
        timeseries_dict: dict[str, TimeSeries] = dict()
        for i in range(len(vects)):
            unit = None
            if units is not None:
                unit = units[i]
            timeseries_dict[channels[i]] = TimeSeries(
                data=vects[i].data,
                unit=unit,
                t0=start_time,
                sample_rate=vects[i].fsample * Hz,
            )
        return TimeSeriesDict(timeseries_dict)
    else:
        raise RuntimeError(
            "virgotools not available in this environment, cannot download online Virgo data"
        )
